Module SVlib
!===============================================================================
!     SVlib module contains subroutines useful for SismoVi data handling
!-------------------------------------------------------------------------------
implicit none

private
public :: SismoViExport
integer, parameter, public :: FINIS=-1, BEGIN=1

Contains
!===============================================================================
Subroutine SismoViExport(operation, trace, array, title, xlabel, ylabel,  &
                         traces, tracestep, samples, timestep, fformat, fname)
!-------------------------------------------------------------------------------
!
! SismoViExport  routine:
!    
!      Writes  files in format readable by SismoVi  signal visualizer.
!      Operation flag  =  1 (BEGIN) / -1 (FINIS) 
!      Export MUST always start with a first call using BEGIN and end with 
!      a last call using FINIS.
!      SINGLE allows for writing single traces of variable length.
!      MULTI  allows for writing an array containing a number of "traces" having 
!      the same length "samples" with trace step "tracestep" and 
!      time step "timestep".
!      "fformat" parameter allows for different file formats (not yet implemented)
!
! Author: G.Seriani, OGS, Trieste 7/10/2004 
!--------------------------------------------------------------------------------
implicit none

character(*), optional :: title, xlabel, ylabel, fformat, fname
integer,      optional :: operation, traces, samples
real,         optional :: trace(:), array(:,:), tracestep, timestep
integer, parameter :: SINGLE=10, MULTI=11     
integer       :: opt, nchout
logical, save :: started=.FALSE.

opt=0; nchout = 20

if(present( trace ))      opt = SINGLE          ! error is raised if both "trace" 
if(present( array ))      opt = opt + MULTI     ! and "array" are present
if(present( operation ))  opt = operation


select case (opt)
  case (BEGIN)
       Call Prolog; started =.TRUE.
  case (SINGLE)
       if(.NOT.started) Call Prolog; started =.TRUE.
       write(nchout,*) "["
       write(nchout,"(G10.4,2H, )") trace
       write(nchout,*) "],"
  case (MULTI)
       if(.NOT.started) Call Prolog; started =.TRUE.
       write(nchout,"(G10.4,2H, )") array
  case (FINIS)
       Call Epilog
  case default
       STOP "Export error"
end select

Contains
   Subroutine Prolog
      OPEN(UNIT=nchout, FILE=fname, FORM='FORMATTED', STATUS='REPLACE')
      write(nchout,*) "{"
      if(present( title  ))  write(nchout,*) "'title': '", title ,"',"
      if(present( xlabel ))  write(nchout,*) "'xlabel':'", xlabel ,"',"
      if(present( ylabel ))  write(nchout,*) "'ylabel':'", ylabel ,"',"
      if(present( traces ))  write(nchout,*) "'traces':" , traces  ,","
      if(present( samples )) write(nchout,*) "'samples':", samples ,","
      if(present( tracestep )) then
                write(nchout,*) "'tracestep':", tracestep ,","
      else
                write(nchout,*) "'tracestep':", 1. ,","
      end if
      if(present( timestep  )) then 
                write(nchout,*) "'timestep':" , timestep  ,","
      else
                write(nchout,*) "'timestep':" , 1. ,","
      end if
      write(nchout,*) "'data':["
   End Subroutine Prolog
   
   Subroutine Epilog
      write(nchout,*) "]"
      write(nchout,*) "}"
      CLOSE(nchout)
   End Subroutine Epilog

End Subroutine SismoViExport   
End Module SVlib    
